#!/bin/bash -i

###Converted from AttractEasyModel by easy2model 1.3
### Generated by ATTRACT shell script generator version 0.8

set -u -e
if [ ! -f $ATTRACTDIR/attract ]; then
  echo 'Cannot find the ATTRACT binary in $ATTRACTDIR='\"$ATTRACTDIR\"
  echo 'Please install ATTRACT following the installation instructions'
  echo 'ATTRACT is available at: http://www.attract.ph.tum.de/services/ATTRACT/attract.tgz'
  exit 1
fi
if [ ! -f $ATTRACTDIR/../version ] ||  [ `awk '{print ($1 < 0.4)}' $ATTRACTDIR/../version` -eq 1 ]; then
  echo 'Your ATTRACT version is too old to run this protocol. Please download and install the latest version of ATTRACT'
  echo 'ATTRACT is available at: http://www.attract.ph.tum.de/services/ATTRACT/attract.tgz'
  exit 1
fi
trap "kill -- -$BASHPID; attractclean" ERR EXIT
$ATTRACTDIR/shm-clean

rm -rf result.dat result.pdb result.lrmsd result.irmsd result.fnat result.interface >& /dev/null
function attractclean {
  $ATTRACTDIR/shm-clean
}

#name of the run
name=protein_rigid_rna_3_hm

#docking parameters
params="$ATTRACTDIR/../attract.par receptorr.pdb ligandr.pdb --fix-receptor --modes hm-all.dat --gravity 2"
scoreparams="$ATTRACTDIR/../attract.par receptorr.pdb ligandr.pdb --score --fix-receptor --modes hm-all.dat"

#grid parameters
gridparams=" --grid 1 receptorgrid.gridheader"

#parallelization parameters
parals="--np 4 --chunks 4"

#see if pypy is installed
PYPY=python2
command -v pypy >/dev/null 2>&1 && PYPY=pypy

if [ 1 -eq 1 ]; then ### move and change to disable parts of the protocol

echo '**************************************************************'
echo 'Reduce partner PDBs...'
echo '**************************************************************'
python2 $ATTRACTDIR/../allatom/aareduce.py receptor.pdb receptor-aa.pdb --chain A --dumppatch --pdb2pqr > receptor.mapping
python2 $ATTRACTDIR/../allatom/aareduce.py receptor-aa.pdb receptor-heavy.pdb --heavy --chain A --readpatch  > /dev/null
python2 $ATTRACTTOOLS/reduce.py receptor-aa.pdb receptorr.pdb --chain A > /dev/null
python2 $ATTRACTDIR/../allatom/aareduce.py ligand.pdb ligand-aa.pdb --chain B --rna --dumppatch --nalib > ligand.mapping
python2 $ATTRACTDIR/../allatom/aareduce.py ligand-aa.pdb ligand-heavy.pdb --heavy --chain B --rna --readpatch  > /dev/null
python2 $ATTRACTTOOLS/reduce.py ligand-aa.pdb ligandr.pdb --chain B --rna > /dev/null

echo '**************************************************************'
echo 'Assemble modes file...'
echo '**************************************************************'
cat /dev/null > hm-all.dat
cat /dev/null > hm-all-aa.dat
cat /dev/null > hm-all-heavy.dat

echo 0 >> hm-all.dat
echo 0 >> hm-all-aa.dat
echo 0 >> hm-all-heavy.dat
python2 $ATTRACTTOOLS/modes.py ligandr.pdb --rna --aacontact --aapdb ligand-aa.pdb 3 > partner2-hm.dat
python2 $ATTRACTTOOLS/modes.py ligand-aa.pdb --rna --aacontact --aapdb ligand-aa.pdb 3 > partner2-hm-aa.dat
python2 $ATTRACTTOOLS/modes.py ligand-heavy.pdb --rna --aacontact --aapdb ligand-aa.pdb 3 > partner2-hm-heavy.dat
cat partner2-hm.dat >> hm-all.dat
cat partner2-hm-aa.dat >> hm-all-aa.dat
cat partner2-hm-heavy.dat >> hm-all-heavy.dat

echo '**************************************************************'
echo 'Reduce reference PDBs...'
echo '**************************************************************'
python2 $ATTRACTDIR/../allatom/aareduce.py receptor-rmsd.pdb refe-rmsd-1.pdb --heavy --pdb2pqr > /dev/null
python2 $ATTRACTDIR/../allatom/aareduce.py ligand-rmsd.pdb refe-rmsd-2.pdb --heavy --nalib  --rna > /dev/null

echo '**************************************************************'
echo 'Generate starting structures...'
echo '**************************************************************'
cat $ATTRACTDIR/../rotation.dat > rotation.dat
$ATTRACTDIR/translate receptorr.pdb ligandr.pdb > translate.dat
$ATTRACTDIR/systsearch > systsearch.dat
start=systsearch.dat

echo '**************************************************************'
echo 'calculate receptorgrid grid'
echo '**************************************************************'
awk '{print substr($0,58,2)}' ligandr.pdb | sort -nu > receptorgrid.alphabet
$ATTRACTDIR/make-grid-omp receptorr.pdb $ATTRACTDIR/../attract.par 5.0 7.0 receptorgrid.gridheader  --shm --alphabet receptorgrid.alphabet

echo '**************************************************************'
echo 'Docking'
echo '**************************************************************'

echo '**************************************************************'
echo '1st minimization'
echo '**************************************************************'
python2 $ATTRACTDIR/../protocols/attract.py $start $params $gridparams --vmax 1000 $parals --output out_$name.dat

echo '**************************************************************'
echo 'Final rescoring'
echo '**************************************************************'
python2 $ATTRACTDIR/../protocols/attract.py out_$name.dat $scoreparams --rcut 50.0 $parals --output out_$name.score

echo '**************************************************************'
echo 'Merge the scores with the structures'
echo '**************************************************************'
$PYPY $ATTRACTTOOLS/fill-energies.py out_$name.dat out_$name.score > out_$name-scored.dat

echo '**************************************************************'
echo 'Sort structures'
echo '**************************************************************'
$PYPY $ATTRACTTOOLS/sort.py out_$name-scored.dat > out_$name-sorted.dat

echo '**************************************************************'
echo 'Remove redundant structures'
echo '**************************************************************'
$ATTRACTDIR/deredundant out_$name-sorted.dat 2 --modes 0 3 | $PYPY $ATTRACTTOOLS/fill-deredundant.py /dev/stdin out_$name-sorted.dat > out_$name-sorted-dr.dat

echo '**************************************************************'
echo 'Soft-link the final results'
echo '**************************************************************'
ln -s out_$name-sorted-dr.dat result.dat

echo '**************************************************************'
echo 'collect top 50 structures:'
echo '**************************************************************'
$ATTRACTTOOLS/top out_$name-sorted-dr.dat 50 > out_$name-top50.dat
$PYPY $ATTRACTTOOLS/demode.py out_$name-top50.dat  > out_$name-top50.dat-demode
$ATTRACTDIR/collect out_$name-top50.dat-demode receptor-aa.pdb ligand-aa.pdb > out_$name-top50.pdb
ln -s out_$name-top50.pdb result.pdb

echo '**************************************************************'
echo 'calculate all-atom interface RMSD'
echo '**************************************************************'
python2 $ATTRACTDIR/irmsd.py out_$name-sorted-dr.dat receptor-heavy.pdb refe-rmsd-1.pdb ligand-heavy.pdb refe-rmsd-2.pdb --modes hm-all-heavy.dat --allatoms > out_$name-sorted-dr.irmsd
ln -s out_$name-sorted-dr.irmsd result.irmsd

attractclean

fi ### move to disable parts of the protocol
